@@ -1,6 +1,6 @@
 /*
  * ****************************************************************************
- * Copyright (c) 2013-2022, PyInstaller Development Team.
+ * Copyright (c) 2013-2023, PyInstaller Development Team.
  *
  * Distributed under the terms of the GNU General Public License (version 2
  * or later) with exception for distributing the bootloader.
@@ -32,48 +32,14 @@
 int pyvers = 0;
 
 /*
- * Return pointer to next toc entry.
+ * Return pointer to the next TOC entry.
  */
-TOC *
-pyi_arch_increment_toc_ptr(const ARCHIVE_STATUS *status, const TOC* ptoc)
+const TOC *
+pyi_arch_increment_toc_ptr(const ARCHIVE_STATUS *status, const TOC *ptoc)
 {
-    TOC *result = (TOC*)((char *)ptoc + ptoc->structlen);
-
-    if (result < status->tocbuff) {
-        FATALERROR("Cannot read Table of Contents.\n");
-        return status->tocend;
-    }
-    return result;
+    return (const TOC *)((const char *)ptoc + ptoc->structlen);
 }
 
-/*
- * Open archive file if needed
- */
-static int
-pyi_arch_open_fp(ARCHIVE_STATUS *status)
-{
-    if (status->fp == NULL) {
-        status->fp = pyi_path_fopen(status->archivename, "rb");
-
-        if (status->fp == NULL) {
-            return -1;
-        }
-    }
-    return 0;
-}
-
-/*
- * Close archive file
- * File should close after unused to avoid locking
- */
-static void
-pyi_arch_close_fp(ARCHIVE_STATUS *status)
-{
-    if (status->fp != NULL) {
-        pyi_path_fclose(status->fp);
-        status->fp = NULL;
-    }
-}
 
 /*
  * Helper for pyi_arch_extract/pyi_arch_extract2fs that extracts a
@@ -82,7 +48,7 @@ pyi_arch_close_fp(ARCHIVE_STATUS *status)
  * to be valid.
  */
 static int
-_pyi_arch_extract_compressed(ARCHIVE_STATUS *status, TOC *ptoc, FILE *out_fp, unsigned char *out_ptr)
+_pyi_arch_extract_compressed(FILE *archive_fp, const TOC *ptoc, FILE *out_fp, unsigned char *out_ptr)
 {
     const size_t CHUNK_SIZE = 8192;
     unsigned char *buffer_in = NULL;
@@ -120,7 +86,7 @@ _pyi_arch_extract_compressed(ARCHIVE_STATUS *status, TOC *ptoc, FILE *out_fp, un
     do {
         /* Read chunk to input buffer */
         size_t chunk_size = (CHUNK_SIZE < remaining_size) ? CHUNK_SIZE : (size_t)remaining_size;
-        if (fread(buffer_in, 1, chunk_size, status->fp) != chunk_size || ferror(status->fp)) {
+        if (fread(buffer_in, 1, chunk_size, archive_fp) != chunk_size || ferror(archive_fp)) {
             rc = -1;
             goto cleanup;
         }
@@ -180,7 +146,7 @@ cleanup:
  * the archive into the provided file handle.
  */
 static int
-_pyi_arch_extract2fs_uncompressed(ARCHIVE_STATUS *status, TOC *ptoc, FILE *out)
+_pyi_arch_extract2fs_uncompressed(FILE *archive_fp, const TOC *ptoc, FILE *out_fp)
 {
     const size_t CHUNK_SIZE = 8192;
     unsigned char *buffer;
@@ -198,12 +164,12 @@ _pyi_arch_extract2fs_uncompressed(ARCHIVE_STATUS *status, TOC *ptoc, FILE *out)
     remaining_size = ptoc->ulen;
     while (remaining_size > 0) {
         size_t chunk_size = (CHUNK_SIZE < remaining_size) ? CHUNK_SIZE : (size_t)remaining_size;
-        if (fread(buffer, chunk_size, 1, status->fp) < 1) {
+        if (fread(buffer, chunk_size, 1, archive_fp) < 1) {
             FATAL_PERROR("fread", "Failed to extract %s: failed to read data chunk!\n", ptoc->name);
             rc = -1;
             break;
         }
-        if (fwrite(buffer, chunk_size, 1, out) < 1) {
+        if (fwrite(buffer, chunk_size, 1, out_fp) < 1) {
             FATAL_PERROR("fwrite", "Failed to extract %s: failed to write data chunk!\n", ptoc->name);
             rc = -1;
             break;
@@ -219,18 +185,18 @@ _pyi_arch_extract2fs_uncompressed(ARCHIVE_STATUS *status, TOC *ptoc, FILE *out)
  * the archive into the provided (pre-allocated) buffer.
  */
 static int
-_pyi_arch_extract_uncompressed(ARCHIVE_STATUS *status, TOC *ptoc, unsigned char *out)
+_pyi_arch_extract_uncompressed(FILE *archive_fp, const TOC *ptoc, unsigned char *out_buf)
 {
     const size_t CHUNK_SIZE = 8192;
     unsigned char *buffer;
     uint64_t remaining_size;
 
     /* Read the file into buffer, chunk by chunk */
-    buffer = out;
+    buffer = out_buf;
     remaining_size = ptoc->ulen;
     while (remaining_size > 0) {
         size_t chunk_size = (CHUNK_SIZE < remaining_size) ? CHUNK_SIZE : (size_t)remaining_size;
-        if (fread(buffer, chunk_size, 1, status->fp) < 1) {
+        if (fread(buffer, chunk_size, 1, archive_fp) < 1) {
             FATAL_PERROR("fread", "Failed to extract %s: failed to read data chunk!\n", ptoc->name);
             return -1;
         }
@@ -245,20 +211,22 @@ _pyi_arch_extract_uncompressed(ARCHIVE_STATUS *status, TOC *ptoc, unsigned char
  * Returns pointer to the data (must be freed).
  */
 unsigned char *
-pyi_arch_extract(ARCHIVE_STATUS *status, TOC *ptoc)
+pyi_arch_extract(const ARCHIVE_STATUS *status, const TOC *ptoc)
 {
+    FILE *archive_fp = NULL;
     unsigned char *data = NULL;
     int rc = 0;
 
     /* Open archive (source) file... */
-    if (pyi_arch_open_fp(status) != 0) {
+    archive_fp = pyi_path_fopen(status->archivename, "rb");
+    if (archive_fp == NULL) {
         FATALERROR("Failed to extract %s: failed to open archive file!\n", ptoc->name);
         return NULL;
     }
     /* ... and seek to the beginning of entry's data */
-    if (pyi_fseek(status->fp, status->pkgstart + ptoc->pos, SEEK_SET) < 0) {
+    if (pyi_fseek(archive_fp, status->pkgstart + ptoc->pos, SEEK_SET) < 0) {
         FATAL_PERROR("fseek", "Failed to extract %s: failed to seek to the entry's data!\n", ptoc->name);
-        return NULL;
+        goto cleanup;
     }
 
     /* Allocate the data buffer */
@@ -270,9 +238,9 @@ pyi_arch_extract(ARCHIVE_STATUS *status, TOC *ptoc)
 
     /* Extract */
     if (ptoc->cflag == '\1') {
-        rc = _pyi_arch_extract_compressed(status, ptoc, NULL, data);
+        rc = _pyi_arch_extract_compressed(archive_fp, ptoc, NULL, data);
     } else {
-        rc = _pyi_arch_extract_uncompressed(status, ptoc, data);
+        rc = _pyi_arch_extract_uncompressed(archive_fp, ptoc, data);
     }
     if (rc != 0) {
         free(data);
@@ -280,40 +248,87 @@ pyi_arch_extract(ARCHIVE_STATUS *status, TOC *ptoc)
     }
 
 cleanup:
-    pyi_arch_close_fp(status);
+    fclose(archive_fp);
 
     return data;
 }
 
 /*
- * Extract an archive entry into file on the filesystem.
- * The path is relative to the directory the archive is in.
+ * Create/extract symbolic link from the archive.
+ */
+static int
+pyi_arch_create_symlink(const ARCHIVE_STATUS *status, const TOC *ptoc)
+{
+    char *link_target = NULL;
+    char link_name[PATH_MAX];
+    int rc = -1;
+
+    /* Extract symlink target */
+    link_target = (char *) pyi_arch_extract(status, ptoc);
+    if (!link_target) {
+        goto cleanup;
+    }
+
+    /* Ensure parent path exists */
+    if (pyi_create_parent_directory(status->temppath, ptoc->name) < 0) {
+        goto cleanup;
+    }
+
+    /* Create the symbolic link */
+    if (snprintf(link_name, PATH_MAX, "%s%c%s", status->temppath, PYI_SEP, ptoc->name) >= PATH_MAX) {
+        goto cleanup;
+    }
+    rc = pyi_path_mksymlink(link_target, link_name);
+
+cleanup:
+    free(link_target);
+
+    return rc;
+}
+
+/*
+ * Extract an archive entry into file in the temporary directory.
+ * The temporary directory must be initialized via `pyi_arch_create_tempdir`
+ * before this function is called.
  */
 int
-pyi_arch_extract2fs(ARCHIVE_STATUS *status, TOC *ptoc)
+pyi_arch_extract2fs(const ARCHIVE_STATUS *status, const TOC *ptoc)
 {
-    FILE *out = NULL;
+    FILE *archive_fp = NULL;
+    FILE *out_fp = NULL;
     int rc = 0;
 
-    /* Ensure that tmp dir _MEIPASSxxx exists... */
-    if (pyi_create_temp_path(status) == -1) {
+    /* Temporary directory must be initialized before calling this function */
+    if (status->has_temp_directory != true) {
+        FATALERROR("pyi_arch_extract2fs was called before temporary directory was initialized!\n");
         return -1;
     }
-    /* ... and open target file */
-    out = pyi_open_target(status->temppath, ptoc->name);
-    if (out == NULL) {
+
+    /* Handle symbolic links */
+    if (ptoc->typcd == ARCHIVE_ITEM_SYMLINK) {
+        rc = pyi_arch_create_symlink(status, ptoc);
+        if (rc < 0) {
+            FATALERROR("Failed to create symbolic link %s!\n", ptoc->name);
+        }
+        return rc;
+    }
+
+    /* Open target file */
+    out_fp = pyi_open_target_file(status->temppath, ptoc->name);
+    if (out_fp == NULL) {
         FATAL_PERROR("fopen", "Failed to extract %s: failed to open target file!\n", ptoc->name);
         return -1;
     }
 
     /* Open archive (source) file... */
-    if (pyi_arch_open_fp(status) != 0) {
+    archive_fp = pyi_path_fopen(status->archivename, "rb");
+    if (archive_fp == NULL) {
         FATALERROR("Failed to extract %s: failed to open archive file!\n", ptoc->name);
         rc = -1;
         goto cleanup;
     }
     /* ... and seek to the beginning of entry's data */
-    if (pyi_fseek(status->fp, status->pkgstart + ptoc->pos, SEEK_SET) < 0) {
+    if (pyi_fseek(archive_fp, status->pkgstart + ptoc->pos, SEEK_SET) < 0) {
         FATAL_PERROR("fseek", "Failed to extract %s: failed to seek to the entry's data!\n", ptoc->name);
         rc = -1;
         goto cleanup;
@@ -321,17 +336,24 @@ pyi_arch_extract2fs(ARCHIVE_STATUS *status, TOC *ptoc)
 
     /* Extract */
     if (ptoc->cflag == '\1') {
-        rc = _pyi_arch_extract_compressed(status, ptoc, out, NULL);
+        rc = _pyi_arch_extract_compressed(archive_fp, ptoc, out_fp, NULL);
     } else {
-        rc = _pyi_arch_extract2fs_uncompressed(status, ptoc, out);
+        rc = _pyi_arch_extract2fs_uncompressed(archive_fp, ptoc, out_fp);
     }
 #ifndef WIN32
-    fchmod(fileno(out), S_IRUSR | S_IWUSR | S_IXUSR);
+    if (ptoc->typcd == ARCHIVE_ITEM_BINARY) {
+        fchmod(fileno(out_fp), S_IRUSR | S_IWUSR | S_IXUSR);
+    } else {
+        fchmod(fileno(out_fp), S_IRUSR | S_IWUSR);
+    }
 #endif
 
 cleanup:
-    pyi_arch_close_fp(status);
-    fclose(out);
+    /* Might be NULL if we jumped here due to fopen() failure */
+    if (archive_fp) {
+        fclose(archive_fp);
+    }
+    fclose(out_fp);
 
     return rc;
 }
@@ -371,8 +393,8 @@ _pyi_arch_fix_toc_endianess(ARCHIVE_STATUS *status)
         ptoc->len = pyi_be32toh(ptoc->len);
         ptoc->ulen = pyi_be32toh(ptoc->ulen);
         /* Jump to next entry; with the current entry fixed up, we can
-         * use pyi_arch_increment_toc_ptr() */
-        ptoc = pyi_arch_increment_toc_ptr(status, ptoc);
+         * use non-const equivalent of pyi_arch_increment_toc_ptr() */
+        ptoc = (TOC *)((const char *)ptoc + ptoc->structlen);
     }
 }
 
@@ -383,31 +405,35 @@ _pyi_arch_fix_toc_endianess(ARCHIVE_STATUS *status)
 int
 pyi_arch_open(ARCHIVE_STATUS *status)
 {
+    FILE *archive_fp = NULL;
     uint64_t cookie_pos = 0;
+    int rc = -1;
+
     VS("LOADER: archivename is %s\n", status->archivename);
 
-    /* Physically open the file */
-    if (pyi_arch_open_fp(status) != 0) {
+    /* Open the archive file */
+    archive_fp = pyi_path_fopen(status->archivename, "rb");
+    if (archive_fp == NULL) {
         VS("LOADER: Cannot open archive: %s\n", status->archivename);
         return -1;
     }
 
     /* Search for the embedded archive's cookie */
-    cookie_pos = _pyi_find_pkg_cookie_offset(status->fp);
+    cookie_pos = _pyi_find_pkg_cookie_offset(archive_fp);
     if (cookie_pos == 0) {
         VS("LOADER: Cannot find cookie!\n");
-        return -1;
+        goto cleanup;
     }
     VS("LOADER: Cookie found at offset 0x%" PRIX64 "\n", cookie_pos);
 
     /* Read the cookie */
-    if (pyi_fseek(status->fp, cookie_pos, SEEK_SET) < 0) {
+    if (pyi_fseek(archive_fp, cookie_pos, SEEK_SET) < 0) {
         FATAL_PERROR("fseek", "Failed to seek to cookie position!\n");
-        return -1;
+        goto cleanup;
     }
-    if (fread(&status->cookie, sizeof(COOKIE), 1, status->fp) < 1) {
+    if (fread(&status->cookie, sizeof(COOKIE), 1, archive_fp) < 1) {
         FATAL_PERROR("fread", "Failed to read cookie!\n");
-        return -1;
+        goto cleanup;
     }
     /* Fix endianness of COOKIE fields */
     status->cookie.len = pyi_be32toh(status->cookie.len);
@@ -426,33 +452,35 @@ pyi_arch_open(ARCHIVE_STATUS *status)
     pyvers = pyi_arch_get_pyversion(status);
 
     /* Read in in the table of contents */
-    pyi_fseek(status->fp, status->pkgstart + status->cookie.TOC, SEEK_SET);
-    status->tocbuff = (TOC *) malloc(status->cookie.TOClen);
+    pyi_fseek(archive_fp, status->pkgstart + status->cookie.TOC, SEEK_SET);
+    status->tocbuff = (TOC *)malloc(status->cookie.TOClen);
 
     if (status->tocbuff == NULL) {
         FATAL_PERROR("malloc", "Could not allocate buffer for TOC!\n");
-        return -1;
+        goto cleanup;
     }
 
-    if (fread(status->tocbuff, status->cookie.TOClen, 1, status->fp) < 1) {
+    if (fread(status->tocbuff, status->cookie.TOClen, 1, archive_fp) < 1) {
         FATAL_PERROR("fread", "Could not read full TOC!\n");
-        return -1;
+        goto cleanup;
     }
-    status->tocend = (TOC *) (((char *)status->tocbuff) + status->cookie.TOClen);
+    status->tocend = (const TOC *)(((const char *)status->tocbuff) + status->cookie.TOClen);
 
     /* Check input file is still ok (should be). */
-    if (ferror(status->fp)) {
+    if (ferror(archive_fp)) {
         FATALERROR("Error on file.\n");
-        return -1;
+        goto cleanup;
     }
 
     /* Fix the endianness of the fields in the TOC entries */
     _pyi_arch_fix_toc_endianess(status);
 
-    /* Close file handler
-     * if file not close here it will be close in pyi_arch_status_free */
-    pyi_arch_close_fp(status);
-    return 0;
+    rc = 0; /* Succeeded */
+
+cleanup:
+    fclose(archive_fp);
+
+    return rc;
 }
 
 /* Setup the archive with python modules and the paths required by rest of
@@ -460,7 +488,7 @@ pyi_arch_open(ARCHIVE_STATUS *status)
  * Sets f_archivename, f_homepath, f_mainpath
  */
 bool
-pyi_arch_setup(ARCHIVE_STATUS *status, char const * archive_path, char const * executable_path)
+pyi_arch_setup(ARCHIVE_STATUS *status, char const *archive_path, char const *executable_path)
 {
     /* Copy archive path and executable path */
     if (snprintf(status->archivename, PATH_MAX, "%s", archive_path) >= PATH_MAX) {
@@ -469,8 +497,39 @@ pyi_arch_setup(ARCHIVE_STATUS *status, char const * archive_path, char const * e
     if (snprintf(status->executablename, PATH_MAX, "%s", executable_path) >= PATH_MAX) {
         return false;
     }
-    /* Set homepath to where the archive is */
-    pyi_path_dirname(status->homepath, archive_path);
+
+    /* Try to open the archive with given archive and executable path */
+    if (pyi_arch_open(status)) {
+        return false;
+    }
+
+    /* Set homepath (a.k.a. sys._MEIPASS) */
+    char executable_dir[PATH_MAX];
+    pyi_path_dirname(executable_dir, executable_path);
+#if defined(__APPLE__)
+    size_t executable_dir_len;
+    executable_dir_len = strnlen(executable_dir, PATH_MAX);
+    bool is_macos_app_bundle = executable_dir_len > 19 && strncmp(executable_dir + executable_dir_len - 19, ".app/Contents/MacOS", 19) == 0;
+#else
+    bool is_macos_app_bundle = false;
+#endif
+    if (is_macos_app_bundle) {
+        /* macOS .app bundle; relocate homepath from Contents/MacOS
+         * directory to Contents/Frameworks */
+        char contents_dir[PATH_MAX];
+        pyi_path_dirname(contents_dir, executable_dir);
+        pyi_path_join(status->homepath, contents_dir, "Frameworks");
+    } else {
+        const char *contents_directory = pyi_arch_get_option(status, "pyi-contents-directory");
+        if (!contents_directory) {
+            FATALERROR("pyi-contents-directory option not found in onedir bundle archive!");
+            return false;
+        }
+        char root_path[PATH_MAX];
+        pyi_path_dirname(root_path, archive_path);
+        pyi_path_join(status->homepath, root_path, contents_directory);
+    }
+
     /*
      * Initial value of mainpath is homepath. It might be overridden
      * by temppath if it is available.
@@ -478,41 +537,14 @@ pyi_arch_setup(ARCHIVE_STATUS *status, char const * archive_path, char const * e
     status->has_temp_directory = false;
     strcpy(status->mainpath, status->homepath);
 
-    /* Open the archive */
-    if (pyi_arch_open(status)) {
-        /* If this is not an archive, we MUST close the file, */
-        /* otherwise the open file-handle will be reused when */
-        /* testing the next file. */
-        pyi_arch_close_fp(status);
-        return false;
-    }
     return true;
 }
 
-/*
- * external API for iterating TOCs
- */
-TOC *
-getFirstTocEntry(ARCHIVE_STATUS *status)
-{
-    return status->tocbuff;
-}
-TOC *
-getNextTocEntry(ARCHIVE_STATUS *status, TOC *entry)
-{
-    TOC *rslt = (TOC*)((char *)entry + entry->structlen);
-
-    if (rslt >= status->tocend) {
-        return NULL;
-    }
-    return rslt;
-}
-
 /*
  * Helpers for embedders.
  */
 int
-pyi_arch_get_pyversion(ARCHIVE_STATUS *status)
+pyi_arch_get_pyversion(const ARCHIVE_STATUS *status)
 {
     return status->cookie.pyvers;
 }
@@ -521,9 +553,10 @@ pyi_arch_get_pyversion(ARCHIVE_STATUS *status)
  * Allocate memory for archive status.
  */
 ARCHIVE_STATUS *
-pyi_arch_status_new() {
+pyi_arch_status_new()
+{
     ARCHIVE_STATUS *archive_status;
-    archive_status = (ARCHIVE_STATUS *) calloc(1, sizeof(ARCHIVE_STATUS));
+    archive_status = (ARCHIVE_STATUS *)calloc(1, sizeof(ARCHIVE_STATUS));
     if (archive_status == NULL) {
         FATAL_PERROR("calloc", "Cannot allocate memory for ARCHIVE_STATUS\n");
     }
@@ -536,17 +569,17 @@ pyi_arch_status_new() {
 void
 pyi_arch_status_free(ARCHIVE_STATUS *archive_status)
 {
-    if (archive_status != NULL) {
-        VS("LOADER: Freeing archive status for %s\n", archive_status->archivename);
-
-        /* Free the TOC memory from the archive status first. */
-        if (archive_status->tocbuff != NULL) {
-            free(archive_status->tocbuff);
-        }
-        /* Close file handler */
-        pyi_arch_close_fp(archive_status);
-        free(archive_status);
+    if (archive_status == NULL) {
+        return;
     }
+
+    VS("LOADER: Freeing archive status for %s\n", archive_status->archivename);
+
+    /* Free the TOC buffer */
+    free(archive_status->tocbuff);
+
+    /* Free the structure itself */
+    free(archive_status);
 }
 
 /*
@@ -557,12 +590,12 @@ pyi_arch_status_free(ARCHIVE_STATUS *archive_status)
  * The string returned is owned by the ARCHIVE_STATUS; the caller is NOT responsible
  * for freeing it.
  */
-char *
-pyi_arch_get_option(const ARCHIVE_STATUS * status, char * optname)
+const char *
+pyi_arch_get_option(const ARCHIVE_STATUS *status, const char *optname)
 {
     /* TODO: option-cache? */
     size_t optlen;
-    TOC *ptoc = status->tocbuff;
+    const TOC *ptoc = status->tocbuff;
 
     optlen = strlen(optname);
 
@@ -577,7 +610,6 @@ pyi_arch_get_option(const ARCHIVE_STATUS * status, char * optname)
                     /* No option value, just return the empty string. */
                     return ptoc->name + optlen;
                 }
-
             }
         }
     }
@@ -587,10 +619,10 @@ pyi_arch_get_option(const ARCHIVE_STATUS * status, char * optname)
 /*
  * Find a TOC entry by its name and return it.
  */
-TOC *
-pyi_arch_find_by_name(ARCHIVE_STATUS *status, const char *name)
+const TOC *
+pyi_arch_find_by_name(const ARCHIVE_STATUS *status, const char *name)
 {
-    TOC *ptoc = status->tocbuff;
+    const TOC *ptoc = status->tocbuff;
 
     while (ptoc < status->tocend) {
         if (strcmp(ptoc->name, name) == 0) {
@@ -600,3 +632,34 @@ pyi_arch_find_by_name(ARCHIVE_STATUS *status, const char *name)
     }
     return NULL;
 }
+
+
+/*
+ * Creates a temporary directory for the ARCHIVE_STATUS.
+ */
+int
+pyi_arch_create_tempdir(ARCHIVE_STATUS *status)
+{
+    const char *runtime_tmpdir = NULL;
+
+    /* No-op if already initialized */
+    if (status->has_temp_directory == true) {
+        return 0;
+    }
+
+    /* Check for custom run-time temporary directory options */
+    runtime_tmpdir = pyi_arch_get_option(status, "pyi-runtime-tmpdir");
+    if (runtime_tmpdir != NULL) {
+        VS("LOADER: Found runtime-tmpdir %s\n", runtime_tmpdir);
+    }
+
+    if (!pyi_create_tempdir(status->temppath, runtime_tmpdir)) {
+        FATALERROR("Cannot create temporary directory!\n");
+        return -1;
+    }
+
+    /* Set flag that temp directory is created and available. */
+    status->has_temp_directory = true;
+
+    return 0;
+}
