@@ -25,7 +25,6 @@
     #include <langinfo.h> /* CODESET, nl_langinfo */
     #include <stdlib.h>   /* malloc */
 #endif
-#include <locale.h>  /* setlocale */
 #include <stdarg.h>
 #include <stddef.h>   /* ptrdiff_t */
 #include <stdio.h>    /* vsnprintf */
@@ -66,6 +65,7 @@ _format_and_check_path(char *buf, const char *fmt, ...)
     };
     va_end(args);
 
+    VS("Checking for file %s\n", buf);
     return stat(buf, &tmp);
 }
 
@@ -96,13 +96,9 @@ _split_dependency_name(char *path, char *filename, const char *item)
 
 /* Copy the dependencies file from a directory to the tempdir */
 static int
-_copy_dependency_from_dir(ARCHIVE_STATUS *status, const char *srcpath, const char *filename)
+_copy_dependency_from_dir(const ARCHIVE_STATUS *status, const char *srcpath, const char *filename)
 {
-    if (pyi_create_temp_path(status) == -1) {
-        return -1;
-    }
-
-    VS("LOADER: Coping file %s to %s\n", srcpath, status->temppath);
+    VS("LOADER: Copying file %s to %s\n", srcpath, status->temppath);
     return pyi_copy_file(srcpath, status->temppath, filename);
 }
 
@@ -124,10 +120,6 @@ _get_archive(ARCHIVE_STATUS *archive_pool[], const char *path)
 
     VS("LOADER: Getting file from archive.\n");
 
-    if (pyi_create_temp_path(archive_pool[SELF]) == -1) {
-        return NULL;
-    }
-
     for (index = 1; archive_pool[index] != NULL; index++) {
         if (strcmp(archive_pool[index]->archivename, path) == 0) {
             VS("LOADER: Archive found: %s\n", path);
@@ -169,7 +161,7 @@ _get_archive(ARCHIVE_STATUS *archive_pool[], const char *path)
 static int
 _extract_dependency_from_archive(ARCHIVE_STATUS *status, const char *filename)
 {
-    TOC *ptoc = status->tocbuff;
+    const TOC *ptoc = status->tocbuff;
 
     VS("LOADER: Extracting dependency %s from archive\n", filename);
 
@@ -196,6 +188,7 @@ _extract_dependency(ARCHIVE_STATUS *archive_pool[], const char *item)
     char archive_path[PATH_MAX];
 
     char dirname[PATH_MAX];
+    char homepath_parent[PATH_MAX];
 
     VS("LOADER: Extracting dependency/reference %s\n", item);
 
@@ -204,16 +197,21 @@ _extract_dependency(ARCHIVE_STATUS *archive_pool[], const char *item)
     }
 
     pyi_path_dirname(dirname, path);
+    pyi_path_dirname(homepath_parent, archive_status->homepath);
+    const char *contents_directory = pyi_arch_get_option(archive_pool[0], "pyi-contents-directory");
+    if (!contents_directory) {
+        FATALERROR("pyi-contents-directory option not found in onedir bundle archive!");
+        return -1;
+    }
 
     /* We need to identify and handle three situations:
      *  1) dependencies are in a onedir archive next to the current onefile archive,
      *  2) dependencies are in a onedir/onefile archive next to the current onedir archive,
      *  3) dependencies are in a onefile archive next to the current onefile archive.
      */
-    VS("LOADER: Checking if file exists\n");
-
+    VS("LOADER: homepath is %s\n", archive_status->homepath);
     /* TODO implement pyi_path_join to accept variable length of arguments for this case. */
-    if (_format_and_check_path(srcpath, "%s%c%s%c%s", archive_status->homepath, PYI_SEP, dirname, PYI_SEP, filename) == 0) {
+    if (_format_and_check_path(srcpath, "%s%c%s%c%s%c%s", homepath_parent, PYI_SEP, dirname, PYI_SEP, contents_directory, PYI_SEP, filename) == 0) {
         VS("LOADER: File %s found, assuming onedir reference\n", srcpath);
 
         if (_copy_dependency_from_dir(archive_status, srcpath, filename) == -1) {
@@ -222,7 +220,7 @@ _extract_dependency(ARCHIVE_STATUS *archive_pool[], const char *item)
         }
         /* TODO implement pyi_path_join to accept variable length of arguments for this case. */
     }
-    else if (_format_and_check_path(srcpath, "%s%c%s%c%s%c%s", archive_status->homepath, PYI_SEP, "..", PYI_SEP, dirname, PYI_SEP, filename) == 0) {
+    else if (_format_and_check_path(srcpath, "%s%c%s%c%s%c%s", homepath_parent, PYI_SEP, dirname, PYI_SEP, contents_directory, PYI_SEP, filename) == 0) {
         VS("LOADER: File %s found, assuming onedir reference\n", srcpath);
 
         if (_copy_dependency_from_dir(archive_status, srcpath, filename) == -1) {
@@ -234,9 +232,9 @@ _extract_dependency(ARCHIVE_STATUS *archive_pool[], const char *item)
         VS("LOADER: File %s not found, assuming onefile reference.\n", srcpath);
 
         /* TODO implement pyi_path_join to accept variable length of arguments for this case. */
-        if ((_format_and_check_path(archive_path, "%s%c%s.pkg", archive_status->homepath, PYI_SEP, path) != 0) &&
-            (_format_and_check_path(archive_path, "%s%c%s.exe", archive_status->homepath, PYI_SEP, path) != 0) &&
-            (_format_and_check_path(archive_path, "%s%c%s", archive_status->homepath, PYI_SEP, path) != 0)) {
+        if ((_format_and_check_path(archive_path, "%s%c%s%c%s.pkg", homepath_parent, PYI_SEP, contents_directory, PYI_SEP, path) != 0) &&
+            (_format_and_check_path(archive_path, "%s%c%s.exe", homepath_parent, PYI_SEP, path) != 0) &&
+            (_format_and_check_path(archive_path, "%s%c%s", homepath_parent, PYI_SEP, path) != 0)) {
             FATALERROR("Archive not found: %s\n", archive_path);
             return -1;
         }
@@ -264,13 +262,13 @@ _extract_dependency(ARCHIVE_STATUS *archive_pool[], const char *item)
  * and a child process will not be required on windows.
  */
 int
-pyi_launch_need_to_extract_binaries(ARCHIVE_STATUS *archive_status)
+pyi_launch_need_to_extract_binaries(const ARCHIVE_STATUS *archive_status)
 {
-    TOC * ptoc = archive_status->tocbuff;
+    const TOC *ptoc = archive_status->tocbuff;
 
     while (ptoc < archive_status->tocend) {
         if (ptoc->typcd == ARCHIVE_ITEM_BINARY || ptoc->typcd == ARCHIVE_ITEM_DATA ||
-            ptoc->typcd == ARCHIVE_ITEM_ZIPFILE) {
+            ptoc->typcd == ARCHIVE_ITEM_ZIPFILE || ptoc->typcd == ARCHIVE_ITEM_SYMLINK) {
             return true;
         }
 
@@ -306,7 +304,7 @@ pyi_launch_extract_binaries(ARCHIVE_STATUS *archive_status, SPLASH_STATUS *splas
      * archive_pool[0] is reserved for the main process, the others for dependencies.
      */
     ARCHIVE_STATUS *archive_pool[_MAX_ARCHIVE_POOL_LEN];
-    TOC * ptoc = archive_status->tocbuff;
+    const TOC *ptoc = archive_status->tocbuff;
 
     /* Clean memory for archive_pool list. */
     memset(archive_pool, 0, _MAX_ARCHIVE_POOL_LEN * sizeof(ARCHIVE_STATUS *));
@@ -318,7 +316,7 @@ pyi_launch_extract_binaries(ARCHIVE_STATUS *archive_status, SPLASH_STATUS *splas
 
     while (ptoc < archive_status->tocend) {
         if (ptoc->typcd == ARCHIVE_ITEM_BINARY || ptoc->typcd == ARCHIVE_ITEM_DATA ||
-            ptoc->typcd == ARCHIVE_ITEM_ZIPFILE) {
+            ptoc->typcd == ARCHIVE_ITEM_ZIPFILE || ptoc->typcd == ARCHIVE_ITEM_SYMLINK) {
             /* 'Splash screen' feature */
             if (update_text) {
                 /* Update the text on the splash screen if one is available */
@@ -465,11 +463,11 @@ _pyi_extract_exception_traceback(PyObject *ptype, PyObject *pvalue,
  * Return non zero on failure
  */
 int
-pyi_launch_run_scripts(ARCHIVE_STATUS *status)
+pyi_launch_run_scripts(const ARCHIVE_STATUS *status)
 {
     unsigned char *data;
     char buf[PATH_MAX];
-    TOC * ptoc = status->tocbuff;
+    const TOC *ptoc = status->tocbuff;
     PyObject *__main__;
     PyObject *__file__;
     PyObject *main_dict;
@@ -514,7 +512,7 @@ pyi_launch_run_scripts(ARCHIVE_STATUS *status)
             }
 
             /* Store the code object to __main__ module's _pyi_main_co
-             * attribute, so it can be retrieved by FrozenImporter,
+             * attribute, so it can be retrieved by PyiFrozenImporter,
              * if necessary. */
             PI_PyObject_SetAttrString(__main__, "_pyi_main_co", code);
 
@@ -622,8 +620,7 @@ pyi_launch_execute(ARCHIVE_STATUS *status)
     /* Load Python DLL */
     if (pyi_pylib_load(status)) {
         return -1;
-    }
-    else {
+    } else {
         /* With this flag Python cleanup will be called. */
         status->is_pylib_loaded = true;
     }
@@ -638,38 +635,19 @@ pyi_launch_execute(ARCHIVE_STATUS *status)
         return -1;
     }
 
-    /* Install zlibs  - now all hooks in place */
-    if (pyi_pylib_install_zlibs(status)) {
+    /* Install PYZ archive */
+    if (pyi_pylib_install_pyz(status)) {
         return -1;
     }
 
-#ifndef WIN32
-
-    /*
-     * On Linux sys.getfilesystemencoding() returns None but should not.
-     * If it's None(NULL), get the filesystem encoding by using direct
-     * C calls and override it with correct value.
-     *
-     * TODO: This may not be needed any more. Please confirm on Linux.
-     */
-    if (!*PI_Py_FileSystemDefaultEncoding) {
-        char *saved_locale, *loc_codeset;
-        saved_locale = strdup(setlocale(LC_CTYPE, NULL));
-        VS("LOADER: LC_CTYPE was %s but resulted in NULL FileSystemDefaultEncoding\n",
-           saved_locale);
-        setlocale(LC_CTYPE, "");
-        loc_codeset = nl_langinfo(CODESET);
-        setlocale(LC_CTYPE, saved_locale);
-        free(saved_locale);
-        VS("LOADER: Setting FileSystemDefaultEncoding to %s (was NULL)\n", loc_codeset);
-        *PI_Py_FileSystemDefaultEncoding = loc_codeset;
-    }
-#endif     /* WIN32 */
-
     /* Run scripts */
     rc = pyi_launch_run_scripts(status);
 
-    VS("LOADER: OK.\n");
+    if (rc == 0) {
+        VS("LOADER: OK.\n");
+    } else {
+        VS("LOADER: ERROR.\n");
+    }
 
     return rc;
 }
