@@ -97,20 +97,6 @@ strnlen(const char *str, size_t n)
 }
 #endif
 
-// some platforms do not provide strndup
-#ifndef HAVE_STRNDUP
-char *
-strndup(const char * str, size_t n)
-{
-    char *ret = NULL;
-    size_t len = strnlen(str, n);
-    ret = (char *)malloc(len + 1);
-    if (ret == NULL) return NULL;
-    ret[len] = '\0';
-    return (char *)memcpy(ret, str, len);
-}
-#endif
-
 
 char *
 pyi_strjoin(const char *first, const char *sep, const char *second){
@@ -241,8 +227,8 @@ pyi_unsetenv(const char *variable)
 #ifdef _WIN32
 
 /* Resolve the runtime tmpdir path and build nested directories */
-wchar_t
-*pyi_build_temp_folder(char *runtime_tmpdir)
+static wchar_t
+*pyi_build_temp_folder(const char *runtime_tmpdir)
 {
     wchar_t *wruntime_tmpdir;
     wchar_t wruntime_tmpdir_expanded[PATH_MAX];
@@ -296,13 +282,13 @@ wchar_t
 
 /* TODO rename function and revisit */
 int
-pyi_get_temp_path(char *buffer, char *runtime_tmpdir)
+pyi_create_tempdir(char *buffer, const char *runtime_tmpdir)
 {
     int i;
     wchar_t *wchar_ret;
     wchar_t prefix[16];
     wchar_t wchar_buffer[PATH_MAX];
-    char *original_tmpdir;
+    char *original_tmpdir = NULL;
     wchar_t *wruntime_tmpdir_abspath;
     DWORD rc;
 
@@ -396,8 +382,8 @@ pyi_test_temp_path(char *buff)
 }
 
 /* TODO merge this function with windows version. */
-static int
-pyi_get_temp_path(char *buff, char *runtime_tmpdir)
+int
+pyi_create_tempdir(char *buff, const char *runtime_tmpdir)
 {
     if (runtime_tmpdir != NULL) {
       strcpy(buff, runtime_tmpdir);
@@ -439,30 +425,6 @@ pyi_get_temp_path(char *buff, char *runtime_tmpdir)
 
 #endif /* ifdef _WIN32 */
 
-/*
- * Creates a temporany directory if it doesn't exists
- * and properly sets the ARCHIVE_STATUS members.
- */
-int
-pyi_create_temp_path(ARCHIVE_STATUS *status)
-{
-    char *runtime_tmpdir = NULL;
-
-    if (status->has_temp_directory != true) {
-        runtime_tmpdir = pyi_arch_get_option(status, "pyi-runtime-tmpdir");
-        if(runtime_tmpdir != NULL) {
-          VS("LOADER: Found runtime-tmpdir %s\n", runtime_tmpdir);
-        }
-
-        if (!pyi_get_temp_path(status->temppath, runtime_tmpdir)) {
-            FATALERROR("INTERNAL ERROR: cannot create temporary directory!\n");
-            return -1;
-        }
-        /* Set flag that temp directory is created and available. */
-        status->has_temp_directory = true;
-    }
-    return 0;
-}
 
 /* TODO merge unix/win versions of remove_one() and pyi_remove_temp_path() */
 #ifdef _WIN32
@@ -481,7 +443,7 @@ remove_one(wchar_t *wfnm, size_t pos, struct _wfinddata_t wfinfo)
         if (!pyi_win32_is_symlink(wfnm)) {
             /* Use recursion to remove subdirectories. */
             pyi_win32_utils_to_utf8(fnm, wfnm, PATH_MAX);
-            pyi_remove_temp_path(fnm);
+            pyi_recursive_rmdir(fnm);
         } else {
             /* Remove only directory link */
             _wrmdir(wfnm);
@@ -494,10 +456,9 @@ remove_one(wchar_t *wfnm, size_t pos, struct _wfinddata_t wfinfo)
     }
 }
 
-/* TODO Find easier and more portable implementation of removing directory recursively. */
-/*     e.g. */
+/* TODO: find easier and more portable implementation of removing directory recursively. */
 void
-pyi_remove_temp_path(const char *dir)
+pyi_recursive_rmdir(const char *dir)
 {
     wchar_t wfnm[PATH_MAX + 1];
     wchar_t wdir[PATH_MAX + 1];
@@ -543,7 +504,7 @@ remove_one(char *pnm, int pos, const char *fnm)
     if (lstat(pnm, &sbuf) == 0) {
         if (S_ISDIR(sbuf.st_mode) ) {
             /* Use recursion to remove subdirectories. */
-            pyi_remove_temp_path(pnm);
+            pyi_recursive_rmdir(pnm);
         }
         else {
             unlink(pnm);
@@ -552,7 +513,7 @@ remove_one(char *pnm, int pos, const char *fnm)
 }
 
 void
-pyi_remove_temp_path(const char *dir)
+pyi_recursive_rmdir(const char *dir)
 {
     char fnm[PATH_MAX + 1];
     DIR *ds;
@@ -604,28 +565,25 @@ _check_strict_unpack_mode ()
 }
 
 /*
- * helper for extract2fs
- * which may try multiple places
+ * Helper that creates parent path for the given filename. The filename
+ * is given as a path prefix (which already exists) and name (which
+ * could be either just a basename or a path relative to the already
+ * existing prefix).
+ *
+ * Returns 0 on success, -1 on failure.
  */
-/* TODO find better name for function. */
-FILE *
-pyi_open_target(const char *path, const char* name_)
+int
+pyi_create_parent_directory(const char *path, const char *name_)
 {
-
-#ifdef _WIN32
-    wchar_t wchar_buffer[PATH_MAX];
-    struct _stat sbuf;
-#else
-    struct stat sbuf;
-#endif
     char fnm[PATH_MAX];
     char name[PATH_MAX];
     char *dir;
     size_t len;
 
+    /* Ensure given components do not exist PATH_MAX */
     if (snprintf(fnm, PATH_MAX, "%s", path) >= PATH_MAX ||
         snprintf(name, PATH_MAX, "%s", name_) >= PATH_MAX) {
-        return NULL;
+        return -1;
     }
 
     len = strlen(fnm);
@@ -635,7 +593,7 @@ pyi_open_target(const char *path, const char* name_)
         len += strlen(dir) + strlen(PYI_SEPSTR);
         /* Check if fnm does not exceed the buffer size */
         if (len >= PATH_MAX-1) {
-            return NULL;
+            return -1;
         }
         strcat(fnm, PYI_SEPSTR);
         strcat(fnm, dir);
@@ -645,24 +603,35 @@ pyi_open_target(const char *path, const char* name_)
             break;
         }
 
-#ifdef _WIN32
-        pyi_win32_utils_from_utf8(wchar_buffer, fnm, PATH_MAX);
-
-        if (_wstat(wchar_buffer, &sbuf) < 0) {
-            pyi_win32_mkdir(wchar_buffer);
+        if (pyi_path_exists(fnm) == 0) {
+            if (pyi_path_mkdir(fnm) < 0) {
+                return -1;
+            }
         }
-#else
-
-        if (stat(fnm, &sbuf) < 0) {
-            mkdir(fnm, 0700);
-        }
-#endif
     }
 
-#ifdef _WIN32
-    pyi_win32_utils_from_utf8(wchar_buffer, fnm, PATH_MAX);
+    return 0;
+}
 
-    if (_wstat(wchar_buffer, &sbuf) == 0) {
+/*
+ * Helper for extract2fs that attempts to open the given filename in
+ * (existing) prefix path, after ensuring that all its parent
+ * directories exist.
+ *
+ * Returns opened FILE handle on success, NULL on failure.
+ */
+FILE *
+pyi_open_target_file(const char *path, const char* name_)
+{
+    char fnm[PATH_MAX];
+
+    /* Merge prefix and name into full path */
+    if (snprintf(fnm, PATH_MAX, "%s%c%s", path, PYI_SEP, name_) >= PATH_MAX) {
+        return NULL;
+    }
+
+    /* Check if file already exists (it should not) */
+    if (pyi_path_exists(fnm) == 1) {
         if (_check_strict_unpack_mode()) {
             OTHERERROR("ERROR: file already exists but should not: %s\n", fnm);
             return NULL;
@@ -670,21 +639,13 @@ pyi_open_target(const char *path, const char* name_)
             OTHERERROR("WARNING: file already exists but should not: %s\n", fnm);
         }
     }
-#else
 
-    if (stat(fnm, &sbuf) == 0) {
-        if (_check_strict_unpack_mode()) {
-            OTHERERROR("ERROR: file already exists but should not: %s\n", fnm);
-            return NULL;
-        } else {
-            OTHERERROR("WARNING: file already exists but should not: %s\n", fnm);
-        }
+    /* Create parent directories, if necessary */
+    if (pyi_create_parent_directory(path, name_) < 0) {
+        return NULL;
     }
-#endif
-    /*
-     * pyi_path_fopen() wraps different fopen names. On Windows it uses
-     * wide-character version of fopen.
-     */
+
+    /* Open the file */
     return pyi_path_fopen(fnm, "wb");
 }
 
@@ -693,7 +654,7 @@ int
 pyi_copy_file(const char *src, const char *dst, const char *filename)
 {
     FILE *in = pyi_path_fopen(src, "rb");
-    FILE *out = pyi_open_target(dst, filename);
+    FILE *out = pyi_open_target_file(dst, filename);
     char buf[4096];
     size_t read_count = 0;
     int error = 0;
@@ -800,6 +761,11 @@ pyi_utils_set_environment(const ARCHIVE_STATUS *status)
 static BOOL WINAPI
 _pyi_win32_console_ctrl(DWORD dwCtrlType)
 {
+    /* Due to different handling of VS() macro in MSVC and mingw gcc, the former requires the name variable
+     * below to be available even in non-debug builds (where VS() is no-op), while the latter complains about
+     * the unused variable. So put everything under ifdef guard to appease both.
+     */
+#if defined(LAUNCH_DEBUG)
     /* https://docs.microsoft.com/en-us/windows/console/handlerroutine */
     static const char *name_map[] = {
         "CTRL_C_EVENT", // 0
@@ -818,6 +784,7 @@ _pyi_win32_console_ctrl(DWORD dwCtrlType)
      * See Remarks section at: https://docs.microsoft.com/en-us/windows/console/setconsolectrlhandler
      */
     VS("LOADER: received console control signal %d (%s)!\n", dwCtrlType, name ? name : "unknown");
+#endif
 
     /* Handle Ctrl+C and Ctrl+Break signals immediately. By returning TRUE, their default handlers
      * (which would call ExitProcess()) are not called, so we are effectively suppressing the signal
