@@ -1,6 +1,6 @@
 /*
  * ****************************************************************************
- * Copyright (c) 2013-2022, PyInstaller Development Team.
+ * Copyright (c) 2013-2023, PyInstaller Development Team.
  *
  * Distributed under the terms of the GNU General Public License (version 2
  * or later) with exception for distributing the bootloader.
@@ -20,7 +20,6 @@
 char * GetWinErrorString(DWORD error_code);
 
 char ** pyi_win32_argv_to_utf8(int argc, wchar_t **wargv);
-wchar_t ** pyi_win32_wargv_from_utf8(int argc, char **argv);
 
 char * pyi_win32_utils_to_utf8(char *buffer, const wchar_t *str, size_t n);
 wchar_t * pyi_win32_utils_from_utf8(wchar_t *buffer, const char *ostr, size_t n);
@@ -33,6 +32,13 @@ int pyi_win32_is_symlink(const wchar_t *path);
 
 int pyi_win32_is_drive_root(const wchar_t *path);
 
+#if !defined(WINDOWED)
+
+void pyi_win32_hide_console();
+void pyi_win32_minimize_console();
+
+#endif
+
 #endif /* ifdef _WIN32 */
 
 #endif  /* UTILS_H */
